Module: OpenStack

Extended by:
ActiveSupport::Autoload
Defined in:
lib/open_stack/base.rb,
lib/open_stack.rb,
lib/open_stack/nova.rb,
lib/open_stack/common.rb,
lib/open_stack/glance.rb,
lib/open_stack/keystone.rb,
lib/open_stack/glance/base.rb,
lib/open_stack/nova/volume.rb,
lib/open_stack/glance/image.rb,
lib/open_stack/nova/compute.rb,
lib/open_stack/keystone/admin.rb,
lib/open_stack/keystone/public.rb,
lib/open_stack/nova/volume/base.rb,
lib/open_stack/nova/compute/base.rb,
lib/open_stack/nova/compute/image.rb,
lib/open_stack/nova/volume/volume.rb,
lib/open_stack/keystone/admin/base.rb,
lib/open_stack/keystone/admin/role.rb,
lib/open_stack/keystone/admin/user.rb,
lib/open_stack/nova/compute/flavor.rb,
lib/open_stack/nova/compute/server.rb,
lib/open_stack/keystone/public/auth.rb,
lib/open_stack/keystone/public/base.rb,
lib/open_stack/nova/compute/network.rb,
lib/open_stack/keystone/admin/tenant.rb,
lib/open_stack/nova/compute/key_pair.rb,
lib/open_stack/keystone/public/tenant.rb,
lib/open_stack/keystone/admin/user_role.rb,
lib/open_stack/nova/compute/base_detail.rb,
lib/open_stack/nova/compute/floating_ip.rb,
lib/open_stack/nova/compute/security_group.rb,
lib/open_stack/nova/compute/floating_ip_pool.rb,
lib/open_stack/nova/compute/volume_attachment.rb,
lib/open_stack/nova/compute/simple_tenant_usage.rb

Overview

This file is part of the OpenStack-ActiveResource

Copyright © 2012 Unidata S.p.A. (Davide Guerri - [email protected])

This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <www.gnu.org/licenses/>.

Defined Under Namespace

Modules: Glance, Keystone, Nova Classes: Base, Common

Constant Summary collapse

DATETIME_FORMAT =
"%Y-%m-%dT%H:%M:%S"
IPV4_REGEX =
/\A(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\Z/
IPV4_CIDR_REGEX =
/\A(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\/(3[0-2]|[1-2][0-9]|[0-9])\Z/